-- Преобразование из любого типа в тип boolean
ToBool = function(value)
	if type(value) == "boolean" then return value
	else
		if value > 0 then return true else return false end
	end
end
--*********************************************************************************************************************************************************
-- исключающее ИЛИ для значений любого типа
XOR = function(bit1, bit2)
	--return bit1 and not bit2 or not bit1 and bit2
	return (ToBool(bit1) or ToBool(bit2)) and not (ToBool(bit1) and ToBool(bit2))
end
--*********************************************************************************************************************************************************
--- инверсия для целочисленного типа
INot = function(value)
	if value == 1 then return 0
	else
		return 1
	end
end
--*********************************************************************************************************************************************************
-- преобразование типа boolean в целочисленный тип
ToInt = function(value)
	if type(value) == "number" then return value
	else
		if value==true then return 1 else return 0 end
	end
end
--*********************************************************************************************************************************************************
-- универсальная функция записи в объект BACnet с заданным именем
function setIO(objname,val)
	if type(objname) ~= "string" then
		for k,v in pairs(objname) do
			local name = tostring(k)
			if AO(name) ~= nil then
				AO(name,ToInt(v),16)
			elseif AV(name) ~= nil then
				AV(name,ToInt(v),16)
			elseif BO(name) ~= nil then
				BO(name,ToInt(v),16)
			elseif BV(name) ~= nil then
				BV(name,ToInt(v),16)
			end
		end
	else
		if AO(objname) ~= nil then
			AO(objname,ToInt(val),16)
		elseif AV(objname) ~= nil then
			AV(objname,ToInt(val),16)
		elseif BO(objname) ~= nil then
			BO(objname,ToInt(val),16)
		elseif BV(objname) ~= nil then
			BV(objname,ToInt(val),16)
		end
	end
end
--*********************************************************************************************************************************************************
---Универсальная функция чтения значения из объекта BACnet с заданным именем
function getIO(objname)
    if AI(objname) ~= nil then
        return AI(objname)
    elseif AO(objname) ~= nil then
        return AO(objname)
    elseif AV(objname) ~= nil then
        return AV(objname)
    elseif BI(objname) ~= nil then
        return ToBool(BI(objname))
    elseif BO(objname) ~= nil then
        return ToBool(BO(objname))
    elseif BV(objname) ~= nil then
        return ToBool(BV(objname))
    end
    return nil
end
--*********************************************************************************************************************************************************
-- Функция обновления значений объекта с включенным сохранением значения
function updateOV(objname, val)
	local oldVal = getIO(objname)
	if oldVal ~= val then
		setIO(objname,val)
	end
end
--*********************************************************************************************************************************************************
--- получение значения объекта BACnet с контролем достоверности значения (выход за допустимый дипапзон значений датчика)
function getTmpSns(name, hiLimit, loLimit)
	local val = getIO(name)
	local almLo = val <= loLimit
	local almHi = val > hiLimit
	return val,almLo or almHi, almLo, almHi
end
--*********************************************************************************************************************************************************
-- компоновка битового слова из массива размерностью 16
Bits2Word = function(bitarr)
	local val = 0
    local k=1
	for k=1,16,1 do
		val = val+ToInt(bitarr[k] or 0)*2^(k-1)
	end
    return val
end
--*********************************************************************************************************************************************************
--- разбор 16-ти битного слова в массив из 16 элементов
Word2Bits = function(word)
	local t=0
	local s=1
	local bitArray = {}
	while word > 0  do
		t = word % 2
		bitArray[s] = t
		word = (word-t)/2
		s = s+1
	end
	for s=s, 16,1 do
		bitArray[s] = 0
	end
	return bitArray
end
--*********************************************************************************************************************************************************
---- функция возвращает true если хотя бы у одного элемента массива ALM значение true
getComAlm = function(ALM)
	res = false
	for _,v in pairs(ALM) do
		res = res or ToBool(v)
	end
	return res
end
--*********************************************************************************************************************************************************
-- Базовый объект. Является родителем для всех объектов библиотеки
function Class(parent)
	local class = {}
	local mClass = {}
	class.__index = class
	mClass.__index = parent 
	class.Super    = parent
	function mClass:__call(...)
		local instance = setmetatable({}, class)
		if type(class.init) == 'function' then
			return instance, instance:init(...)
		end
		return instance
	end
	return setmetatable(class, mClass)
end
--*********************************************************************************************************************************************************
-- Объект. Выдача импульсов с заданным перодом
TICK = Class()
function TICK:init()
	self.StartTime = os.time()
	self.Period = 0
	self.Out = false
end

function TICK:run(sec)
	local curtime = os.time()
	self.Period = sec
	self.Out = false
	if curtime - self.StartTime >= self.Period then
		self.Out = true
		self.StartTime = curtime
	end
	return self.Out
end
--*********************************************************************************************************************************************************
-- Объект. Счетчик времени
RT = Class()
function RT:init(MODE,VALUE) --Mode: 0-часы, 1-минуты, 2-секунды
	self.mode = MODE
	self.minutes = 0
	self.seconds = 0
	self.hours = 0
	self.out = 0
	if MODE == 2 then
		self.seconds = VALUE
		self.out = VALUE
	else
		if MODE == 1 then
			self.minutes = VALUE
			self.out = VALUE
		else
			self.hours = VALUE
			self.out = VALUE
		end
	end
end
  
function RT:run(InValue,Reset) 
	local timer
	local start
	local Val = ToInt(InValue)
	
	if Val == 1 then
		if self.timerStart == nil then self.timerStart = os.time()  - self.seconds end
		timer = os.time()
		self.seconds = timer-self.timerStart 
	end
	
	if self.seconds >59 then self.minutes = self.minutes+1 end
	if self.minutes >59 then self.hours = self.hours+1 end
	
	if self.mode == 2 then 
		self.out = math.floor(self.seconds)
	else
		if self.mode == 1 then
			if self.seconds >59 then self.seconds = 0 self.timerStart=timer  end
			self.out = self.minutes	
		else
			if self.seconds >59 then self.seconds = 0 self.timerStart=timer end
			if self.minutes >59 then self.minutes = 0 end
			self.out = self.hours
		end
	end
	if ToBool(Reset) then 
		self.minutes = 0
		self.seconds = 0
		self.hours = 0
		self.timerStart=nil
	end
	return self.out
end

--- непосредственное задание значения счетчика
function RT:set(VALUE)
		if self.mode == 2 then
			self.timerStart = os.time() - VALUE
			self.seconds = VALUE
		else
			if self.mode == 1 then
				self.minutes = VALUE
				self.timerStart = os.time()
			else
				self.hours = VALUE
				self.minutes = 0
				self.timerStart = os.time()
			end
		end
		self.out = VALUE
end
--*********************************************************************************************************************************************************	
-- детектор фронта дискретного сигнала
TRIG=Class()
function TRIG:init(mode)					--Mode: 0-передний фронт, 1-задний фронт, 2-любое изменение
	self.mode = mode
	self.value = false
	self.out = false
end

function TRIG:run(Value)
	self.out = false
	local Val = ToBool(Value)
	if self.value ~= Val then 
		if self.mode == 0 then
			if Val then self.out = true end
		else
			if self.mode == 1 then
			if not Val then self.out = true end
			else
			self.out = true
			end
		end
	end
	self.value = Val
	return self.out
end	
--*********************************************************************************************************************************************************	
-- Объект. Осциллятор - генерация импульсов заданной длины с заданым периодом
OSC = Class()
function OSC:init()
	self.out = false
	self.In = false
	self.StartTime = 0
	self.dt = 0
	self.trig = TRIG(0)
end
	
function OSC:run(In, PulseTime, Period)
	local CurTime = os.time()
	self.In = ToBool(In)
	self.dt = CurTime - self.StartTime
	if self.trig(self.In) or self.In and self.dt >= Period then
		self.StartTime = os.time()
	end
	self.out = self.In and self.dt < PulseTime
	return self.out
end
--*********************************************************************************************************************************************************	
--   Объект. Аналоговый ПИД-регулятор
PID = Class()
   
function PID:init()
	self.e = 0
	self.G_old = 0
	self.Ti_old = 0
	self.Td_old = 0
	self.Stroke_old = 0
	self.Umin_old = 0
	self.Umax_old = 0
	self.G = 0
	self.Ti = 0
	self.Td = 0
	self.kp = 0
	self.ki = 0
	self.kd = 0
	self.Stroke = 0
	self.Umin = 0
	self.Umax = 0
	self.h  = 1
	self.h_old = 1
	self.eh = 0
	self.yhh = 0
	self.yh = 0
	self.y = 0
	self.dU = 0
	self.DuMax = 0
	self.NonTiBase = 0
	self.started = false
	self.paramchanged = false
	self.calc = 0
	self.out = 0
	self.Tsg = 0
	self.tick = TICK()
end
  
-- задание параметров регулятора
function PID:updatePidParam(par,Ti,Td,Dz, Umax,Umin,StrokeTime,CtrlInt)
	if type(par) == "table" then
		self.G = par.G
		self.Ti = par.Ti
		self.Td = par.Td
		self.h = par.CtrlInt
    self.Dz = par.Dz
		self.Umax = par.Umax
		self.Umin = par.Umin
		self.Stroke = par.Stroke
	else
		self.G = par
		self.Ti = Ti
		self.Td = Td
    self.Dz = Dz
		self.h = CtrlInt
		self.Umax = Umax
		self.Umin = Umin
		self.Stroke = StrokeTime
	end
	if self.Ti ~= self.Ti_old or self.G ~= self.G_old or self.Td ~= self.Td_old or self.h ~= self.h_old or not self.started then      
		self.Ti_old = self.Ti
		self.Td_old = self.Td
		self.G_old = self.G
		self.paramchanged = true
		self.started = true
		self.kp = self.G
		if self.Ti > 0 then
			self.ki = self.G * self.h / self.Ti
		else
			self.ki = 0.0
		end
		self.kd = self.G * self.Td / self.h
	else
		self.paramchanged = false
	end

	if self.Stroke ~= self.Stroke_old or self.Umax ~= self.Umax_old or self.Umin ~= self.Umin_old or not self.started then
		if self.Stroke ~= 0 then
			self.DuMax	=	(self.Umax-self.Umin)*self.h/self.Stroke
		else
			self.DuMax	=	self.Umax
		end
		self.NonTiBase = (self.Umax + self.Umin) / 2.0
		self.Stroke_old = self.Stroke
		self.Umax_old = self.Umax
		self.Umin_old = self.Umin
	end 
end
  
-- основная работа ПИД
function PID:run(par,SP,Mod,TSg)  --Mod: 0-регулятор отключен, 1-регулирование, 2-максимальный выход, 3-минимальный выход
	local e
	local DuMax
	local dU
	local Output

	if type(par) == "table" then
		self.Mv = par.MV
		self.Sp = par.SP
		self.Mode = par.Mod
		self.Tsg = par.TSg
	else
		self.Mv = par
		self.Sp = SP
		self.Mode = Mod
		self.Tsg = TSg
	end
	if self.tick:run(self.h) then
		self.eh	=	self.e
		self.e	=	self.Sp - self.Mv
		self.yhh	=	self.yh
		self.yh	=	self.y
		self.y	=	self.Mv

		local switch_tbl1 = {
		[0] = function(self)  
				self.started = false
				self.out =	self.Tsg
			end,
		[1] = function(self)
			if math.abs(self.e) < self.Dz then 
				self.dU =	0.0
			else	
				if self.Ti > 0.0 then   
					self.dU = self.kp * self.e  + self.ki * (self.e - self.eh) - self.kd * (self.y - 2.0 * self.yh + self.yhh)
					self.dU = math.min(math.max(-self.DuMax, self.dU), self.DuMax)
					self.out =	self.Tsg + self.dU
				else
					self.out =	self.kp * self.e - self.kd * (self.y - self.yh) + self.NonTiBase       
				end 
			end
		end,
		[2] = function(self)
				self.started = false
				self.out =	self.Umax
			end,
		[3] = function(self)
				self.started = false
				self.out =	self.Umin
			end
		}
		switch_tbl1[self.Mode](self)
		self.out = math.min(math.max(self.out, self.Umin), self.Umax)
	else
		local switch_tbl2 = {
		[0] = function(self) self.out =	self.Tsg end,
		[1] = function(self) self.out =	self.out end,
		[2] = function(self) self.out =	self.Umax end,
		[3] = function(self) self.out =	self.Umin end
		}
		switch_tbl2[self.Mode](self)
		self.out = self.out
	end
	return self.out
end 
--*********************************************************************************************************************************************************
-- Объект. ШИМ-регулятор для клапана. Клапан при достижении минимального значения переходит в режим ШИМ с заданной амплитудой и скважностью
PWM = Class()
function PWM:init()
	self.timer_PWM	 = RT(2,0)
	self.out = 0
end
  
function PWM:run(VALVE,PERIOD,MINVALUE)
	local period
	local allow
	local curperiod
	if VALVE<= MINVALUE then
    if VALVE < 0.5 then
      period = 0
    else
      period = PERIOD*VALVE/MINVALUE
      if period < 4 then period = 4 end
    end
		curperiod = self.timer_PWM:run(1,0)
		if curperiod >= PERIOD then self.timer_PWM:set(0) end
		allow = (curperiod <= period)
		if allow == true then self.out = MINVALUE else self.out = 0 end
	else
		self.out = VALVE
	end
	return self.out
end
--*********************************************************************************************************************************************************
--  Объект. Интегральный ПИД-регулятор для 3-з позиционных клапанов
PIDI = Class()
   
function PIDI:init()
	self.e = 0
	self.G_old = 0
	self.Ti_old = 0
	self.Td_old = 0
	self.Stroke_old = 0
	self.G = 0
	self.Ti = 0
	self.Td = 0
	self.kp = 0
	self.ki = 0
	self.kd = 0
	self.Stroke = 0
	self.h  = 1
	self.h_old = 1
	self.eh = 0
	self.yhh = 0
	self.yh = 0
	self.y = 0
	self.dU = 0
	self.DuMax = 0
	self.started = false
	self.paramchanged = false
	self.calc = 0
	self.out = 0
	self.Tsg = 0
	self.tick = TICK()
end

-- задание параметров регулятора
function PIDI:updatePidParam(G,Ti,Td,StrokeTime,CtrlInt)
	self.G = G
	self.Ti = Ti
	self.Td = Td
	self.h = CtrlInt
	if self.Ti ~= self.Ti_old or self.G ~= self.G_old or self.Td ~= self.Td_old or self.h ~= self.h_old or not self.started then      
		self.Ti_old = self.Ti
		self.Td_old = self.Td
		self.G_old = self.G
		self.paramchanged = true
		self.started = true
		self.kp = self.G
		if self.Ti > 0 then
			self.ki = self.G * self.h / self.Ti
		else
			self.ki = 0.0
		end
		self.kd = self.G * self.Td / self.h
	else
		self.paramchanged = false
	end

	self.Stroke = StrokeTime
	if self.Stroke ~= self.Stroke_old or not self.started then
		if self.Stroke ~= 0 then
			self.DuMax	=	100*self.h/self.Stroke
		else
			self.DuMax	=	100*self.h
		end
		self.NonTiBase = 50.0
		self.Stroke_old = self.Stroke
	end 
end
  
-- основная работа регулятора
function PIDI:run(MV,SP,Mod,DZ)  --Mode: 0-отключен, 1-регулирование, 2-на выходе -CtrlInt, 3-на выходе CtrlInt
	local e
	local DuMax
	local dU
	local Output

	self.Mv = MV
	self.Sp = SP
	self.Mode = Mod
	self.Dz = DZ

	if self.tick:run(self.h) then
		self.eh	=	self.e
		self.e	=	self.Sp - self.Mv
		self.yhh	=	self.yh
		self.yh	=	self.y
		self.y	=	self.Mv

		local switch_tbl1 = {
		[0] = function(self)  
				self.started = false
				self.out =	self.Tsg
			end,
		[1] = function(self)
			if math.abs(self.e) < self.Dz then 
				self.dU =	0.0
			else	
				if self.Ti > 0.0 then   
					self.dU = self.kp * self.e  + self.ki * (self.e - self.eh) - self.kd * (self.y - 2.0 * self.yh + self.yhh)
					self.dU = math.min(math.max(-self.DuMax, self.dU), self.DuMax)
					self.out =	self.dU
				else
					self.out =	self.kp * self.e - self.kd * (self.y - self.yh)       
				end 
			end
		end,
		[2] = function(self)
				self.started = false
				self.out =	self.h
			end,
		[3] = function(self)
				self.started = false
				self.out =	-self.h
			end
		}
		switch_tbl1[self.Mode](self)
		self.out = math.min(math.max(self.out, -self.h), self.h)
	else
		local switch_tbl2 = {
		[0] = function(self) self.out =	0 end,
		[1] = function(self) self.out =	self.out end,
		[2] = function(self) self.out =	self.h end,
		[3] = function(self) self.out =	-self.h end
		}
		switch_tbl2[self.Mode](self)
		self.out = self.out
	end
	return self.out
end
--*********************************************************************************************************************************************************
-- Объект. Преобразование выхода PIDI в управление дискретным выходом на клапан (работает в паре с PID)
DOPU = Class()
function DOPU:init(MinPeriod)
	self.minPeriod = MinPeriod
end

function DOPU:run()
	-- наполнение метода
end
--*********************************************************************************************************************************************************
-- Объект. Гистерезис аналоговой величины
HYST=Class()
function HYST:init()
	self.out = 0
end

function HYST:run(Value,act,deact)
	self.on	 = act
	self.off = deact
	if self.out == nil then self.out = 0 end
	
	if self.on > self.off then
		if Value > self.on then self.out = 1
		else if Value < self.off then self.out = 0 end end
	else
		if Value < self.on then self.out = 1
		else if Value > self.off then self.out = 0 end end
	end
	return self.out
end
--*********************************************************************************************************************************************************
--  Объект. Задержка изменения дискретного сигнала (возможны задержки на включение и выключение)
DELAY = Class()
function DELAY:init()					
	self.on	 = 0
	self.off	 = 0
	self.timerON = RT(2,0)
	self.timerOFF = RT(2,0)
	self.value = 0
	self.initON = 0
	self.initOFF = 0
	self.out = 0	
end

function DELAY:run(Value,DelOn, DelOff)
	local a
	local b
	local Val = ToInt(Value)
	self.on = DelOn
	self.off = DelOff
	if Val == 1 and self.value==0 then 
		self.timerON:set(0) 
		self.initON = 1
		self.initOFF = 0
	end
	if Val == 0 and self.value==1 then 
		self.timerOFF:set(0) 
		self.initON = 0
		self.initOFF = 1
	end
	if self.timerON:run(self.initON,0)>= self.on and Val == 1 then self.out = 1 end
	if self.timerOFF:run(self.initOFF,0)>= self.off and Val == 0 then self.out = 0 end
	self.value = Val
		
	return self.out
end	
--*********************************************************************************************************************************************************
-- Объект. Ограничение скорости изменеия сигнала
RAMP=Class()
function RAMP:init(init)					
	if init == nil then init = 0 end
	self.out = init
	self.lasttime = os.time()
end

function RAMP:run(Value, Delta)
	local delta
	local time
	time = os.time()
	self.delta	 = Delta
	delta = (time - self.lasttime)*self.delta
	if Value<self.out then 
		delta = -delta 
	end

	if self.out ~= Value then self.out = self.out+delta end
	if math.abs(self.out-Value)<math.abs(2*delta) then self.out = Value end 
	self.lasttime = time
	return self.out
end
	
-- непосредственное задание значения на входе блока
function RAMP:set(Value)
	self.out = Value
end	
--*********************************************************************************************************************************************************	
-- Функция. Расчет значения функции методом линейной интерполяции
CurveXY=function(X,Y,value,limit)
    local A,B,C,D
	local i = 0;

	if value < X[1] then
		if limit == 1 then
			return Y[1]
		else
			A = Y[2]-Y[1]
			B = X[2]-X[1]
			C = value-X[1]
			D = Y[1]
			return A*C/B+D
		end
	else
		if value > X[#X] then
			if limit == 1 then
				return Y[#Y]
			else
				A = Y[#Y]-Y[#Y-1]
				B = X[#Y]-X[#Y-1]
				C = value-X[#Y-1]
				D = Y[#Y-1]
				return A*C/B+D
			end
		else
			for i = 2, #X-1 do
				if value <= X[i+1] then
					A = Y[i+1]-Y[i]
					B = X[i+1]-X[i]
					C = value-X[i]
					D = Y[i]
					return A*C/B+D
				end
			end
		end
		return Y[1]
	end
end
--*********************************************************************************************************************************************************
--- Объект. Базовый объект аварии
ALARM = Class()
function ALARM:init()
    self.Delay = DELAY()
    self.AlmSta = false
    self.DelOn = 0
    self.AlmVal = true
    self.Ack = true
end

-- задание параметров блока
function ALARM:setParam(AlmVal, DelOn, Ack)
    self.AlmVal = AlmVal
    self.DelOn = DelOn
    self.Ack = Ack
end

function ALARM:run(Sta)
    if ToBool(self.Delay:run(ToBool(Sta) == self.AlmVal, self.DelOn,0)) then
        self.AlmSta = true
    else
        if not self.Ack then self.AlmSta = false end
    end
end

-- сброс аварии
function ALARM:rst()
    self.AlmSta = false
end

--*********************************************************************************************************************************************************
-- Объект. Базовый объект для исполнительного механизма с дискретным управлением
DeviceObj = Class()
function DeviceObj:init()
	self.Md = 0 --Mode: 0 - ручн останов, 1 - ручн запуск, 2 - автоматическое управление
	self.AutCmd = false
	self.AutSta = true
	self.HSta = false
	self.Fbk = false
	self.FbkDelTm = 5
	self.AlmSta = false
	self.DO = false
	self.AL_Fbk = ALARM()
	self.AL_Fbk:setParam(true, 0, true)
	self.AL_AlmSta = ALARM()
	self.AL_AlmSta:setParam(true, 2, true)
	self.AL_Aut = ALARM()
	self.AL_Aut:setParam(false, 0, false)
	self.AL_HStart = ALARM()
	self.AL_HStart:setParam(true, 0, false)
	self.DelayOn = DELAY()
	self.Fault = false -- общий статус аварии
	self.Sta = 0 -- Статус устаройства: 0 - выключено, 1 - включено, 2 - авария
end

-- задание параметров
function DeviceObj:setParam(FbkDelTm)
	self.FbkDelTm = FbkDelTm
end

-- задание положения переключателя Ручн/Авто
function DeviceObj:setAutH(AutSta, HSta)
	self.AutSta = AutSta
	self.HSta = HSta
    self.AL_Aut:run(self.AutSta)
    self.AL_HStart:run(self.HSta)
end

function DeviceObj:run(par, AutCmd, Fbk, AlmSta, EmrgStop, Rst)
	if type(par) == "table" then
		self.Md = par.Md
		self.AutCmd = par.AutCmd
		self.Fbk = par.Fbk
		self.AlmSta = par.AlmSta
		self.EmrgStop = par.EmrgStop
		self.Rst = par.Rst
	else
		self.Md = par
		self.AutCmd = AutCmd
		self.Fbk = Fbk
		self.AlmSta = AlmSta
		self.EmrgStop = EmrgStop
		self.Rst = Rst
	end
	self.DelayOn:run(self.DO,self.FbkDelTm,0)
	self.AL_Fbk:run(ToBool(self.DelayOn.out) and not self.Fbk)
	self.AL_AlmSta:run(self.AlmSta)
	self.Fault = self.AL_Fbk.AlmSta or self.AL_AlmSta.AlmSta or self.AL_Aut.AlmSta
	self.Sta = 2 * ToInt(self.Fault) + 1 * ToInt(self.DO)
    if ToBool(self.Rst) then
        self:AlmRst()
    end
    self.DO = (self.Md == 2 and self.AutCmd or self.Md == 1 ) and not self.Fault and not self.EmrgStop
	return self.DO, self.Sta
end

-- сброс аварий
function DeviceObj:AlmRst()
  self.AL_Fbk:rst()
	self.AL_AlmSta:rst()
end
--*********************************************************************************************************************************************************
-- Объект. Основан на DeviceObj. Управление преобразователем частоты
FCtrl = Class(DeviceObj)
function FCtrl:init()
	self.Super.init(self)
	self.FbkDelTm = 60
	self.Ramp = RAMP(0)
	self.DelayPds = DELAY()
  self.AL_Pds = ALARM()
	self.AL_Pds:setParam(true, 0, true)
	self.intRef = 0
	self.Delta = 1
	self.AO = 0
end

-- задание параметров
function FCtrl:setParam(FbkDelTm, PdsDelTm, Delta)
    self.Super.setParam(self,FbkDelTm)
    self.Delta = Delta
	self.PdsDelTm = PdsDelTm
end

function FCtrl:run(par, AutCmd, Ref, Fbk, AlmSta, Pds, EmrgStop, Rst)
  self.par = {}
	if type(par) == "table" then
		self.par = par
		self.Ref = par.Ref
		self.Pds = par.Pds
	else
		self.par.Md = par
		self.par.AutCmd = AutCmd
		self.par.Fbk = Fbk
		self.par.AlmSta = AlmSta
		self.par.EmrgStop = EmrgStop
		self.par.Rst = Rst
		self.Ref = Ref
		self.Pds = Pds
	end
  self.Super.run(self,self.par)
	self.DelayPds:run(self.DO, self.PdsDelTm,0)
	self.AL_Pds:run(ToBool(self.DelayPds.out) and not self.Pds)
	self.Fault = self.Fault or self.AL_Pds.AlmSta
    if self.DO and not self.Fault then
        self.intRef = math.max(40,Ref)
    else
        self.intRef = 0
    end
    if self.Fault or self.par.EmrgStop then 
		self.Ramp:set(0) 
	end
    self.AO = self.Ramp:run(self.intRef, self.Delta)
    return self.AO, self.DO, self.Sta
end

function FCtrl:AlmRst()
	self.Super.AlmRst(self)
	self.AL_Pds:rst()
end
--*********************************************************************************************************************************************************
-- Объект. Поддержание расхода воздуха
AHU_FlowCtrl = Class()
function AHU_FlowCtrl:init()
end

function AHU_FlowCtrl:setParam()
end

function AHU_FlowCtrl:setPid()
end

function AHU_FlowCtrl:getFlowFromPress()
end

function AHU_FlowCtrl:run()
end
--*********************************************************************************************************************************************************
-- Объект. Управление воздушной заслонкой с дискретным управлением
AHU_DmpDig = Class()
function AHU_DmpDig:init()
	self.AL_Cls = ALARM()
	self.AL_Opn = ALARM()
	self.out = false
	self.OpnSta = false
	self.ClsSta = false
	self.RtOpn = RT(2,0)
	self.RtCls = RT(2,0)
	self.F_Trig = TRIG(0,0)
	self.R_Trig = TRIG(1,0)
	self.Cmd = false
	self.EstTmOpn = 0
	self.EstTmCls = 0
	self.DebugVal = 1
	self.FbType = 0
	self.MovTm = 0
	self.FaCmdDelTm = 10
	self.FaCmd = false
end

-- FbType: 	тип обратной связи: 0-нет, 1-только статус ОТКРЫТО, 2-статусы ОТКРЫТО и ЗАКРЫТО
-- MovTm:	время полного хода заслонки в секундах
-- OpnSta:	статус ОТКРЫТО
-- ClsSta:	статус ЗАКРЫТО
-- Cmd:		команда на открытие
-- Reset:	сброс аварий
function AHU_DmpDig:setParam(FbType, MovTm, FaCmdDelTm)
	if type(FbType) == "table" then
		self.FbType = FbType.FbType
		self.MovTm = FbType.MovTm
		self.FaCmdDelTm = FbType.FaCmdDelTm
	else
		self.FbType = FbType
		self.MovTm = MovTm
		self.FaCmdDelTm = FaCmdDelTm
	end
end

function AHU_DmpDig:run(OpnSta, ClsSta, Ref, Reset)
	self.Cmd = ToBool(Ref)
	
	if self.FbType == 0 then self.OpnSta = self.Cmd else self.OpnSta = ToBool(OpnSta) end
	if self.FbType < 2 then self.ClsSta = not self.Cmd else self.ClsSta = ToBool(ClsSta) end
	
	local OpnEvt = self.F_Trig:run(self.Cmd)
	local ClsEvt = self.R_Trig:run(self.Cmd)
	
	local OpnTmCnt = math.min(self.RtOpn:run(self.Cmd and not self.OpnSta, OpnEvt or ToBool(Reset) and self.AL_Opn.AlmSta),self.MovTm)
	local ClsTmCnt = math.min(self.RtCls:run(not self.Cmd and not self.ClsSta, ClsEvt or ToBool(Reset) and self.AL_Cls.AlmSta),self.MovTm) 
	
	if self.OpnSta then self.RtOpn:set(self.MovTm) OpnTmCnt = self.MovTm end
	if self.ClsSta then self.RtCls:set(self.MovTm) ClsTmCnt = self.MovTm end
	 
	self.EstTmOpn = self.MovTm - OpnTmCnt
	self.EstTmCls = self.MovTm - ClsTmCnt
	
	self.AL_Opn:run(self.Cmd and self.EstTmOpn == 0 and not self.OpnSta)
	self.AL_Cls:run(not self.Cmd and self.EstTmCls == 0 and not self.ClsSta)

	if ToBool(Reset) then
		self.AL_Cls:rst()
		self.AL_Opn:rst()
	end
	
	self.out = self.Cmd and not self.AL_Opn.AlmSta
	if OpnTmCnt >= self.FaCmdDelTm and self.out then
		self.FaCmd = true
	end
	if not self.out then
		self.FaCmd = false
	end
	return self.out, self.FaCmd
end
--*********************************************************************************************************************************************************
-- Объект. Управление воздушной заслонкой с аналоговым управлением
AHU_DmpAn = Class()
function AHU_DmpAn:init()
-- наполнение конструктора
end

-- FbType: 	тип обратной связи: 0-нет, 1-текущее положение заслонки
-- MovTm:	время полного хода заслонки
-- Ref:		сигнал управления приводом
function AHU_DmpAn:setParam(FbType, MovTm, FaCmdDelTm)
    self.FbType = FbType
	self.MovTm = MovTm
	self.FaCmdDelTm = FaCmdDelTm
end

function AHU_DmpAn:run(Ref)
	--наполнение метода
end
--*********************************************************************************************************************************************************
-- Объект. Управление приводом воздушной заслонки с 3-х позиционным управлением
AHU_Dmp3pos = Class()
function AHU_Dmp3pos:init()
-- наполнение конструктора
end

-- FbType: 	тип обратной связи: 0-нет, 1-только статус ОТКРЫТО, 2-статусы ОТКРЫТО и ЗАКРЫТО
-- MovTm:	время полного хода заслонки в секундах
-- OpnSta:	статус ОТКРЫТО
-- ClsSta:	статус ЗАКРЫТО
-- OpnCmd:	команда на открытие
-- ClsCmd:	команда на закрытие
-- Reset:	сброс аварий
function AHU_Dmp3pos:setParam(FbType, MovTm, FaCmdDelTm)
    self.FbType = FbType
	self.MovTm = MovTm
	self.FaCmdDelTm = FaCmdDelTm
end

function AHU_Dmp3pos:run(OpnSta, ClsSta, Ref, Reset)
	--наполнение метода
end
--*********************************************************************************************************************************************************
-- Объект. Управление воздушной заслонкой с дискретным управлением и предварительным нагревом.
AHU_DmpDigPh = Class(AHU_DmpDig)
function AHU_DmpDigPh:init()
	self.Super:init()
	self.PhTime = 30
	self.DelayOpen = DELAY()
	self.PhOut = false
end
--задание параметров
function AHU_DmpDigPh:setParam(FbType, MovTm, FaCmdDelTm, PhTime)
    self.Super:setParam(FbType, MovTm, FaCmdDelTm)
    self.PhTime = PhTime
end

function AHU_DmpDigPh:run(OpnSta, ClsSta, Ref, Reset, summerSta)
	local startOpen = false
	if ToBool(summerSta) then
		self.PhOut = false
		startOpen = Ref
	else
		self.DelayOpen:run(Ref, self.PhTime, 0)
		startOpen = ToBool(self.DelayOpen.out)
		self.PhOut = Ref and not startOpen
	end
    self.Super:run(OpnSta, ClsSta, startOpen, Reset)
    return self.out, self.FaCmd, self.PhOut
end
--*********************************************************************************************************************************************************
-- Объект. Контроль засоренности фильтра
AHU_FltDig = Class()
function AHU_FltDig:init()
	self.AL_PDS = ALARM()
	self.AL_PDS:setParam(true,0,false)
end

function AHU_FltDig:run(PdsVal)
	self.AL_PDS:run(ToBool(PdsVal))
	return self.AL_PDS.AlmSta
end
--*********************************************************************************************************************************************************
-- Объект. Определение текущего сезона: ЗИМА или ЛЕТО
AHU_Summer = Class()
function AHU_Summer:init()
    self.out = false
    self.hyst = HYST()
    self.SwToSummerVal = 14
    self.SwToWinterVal = 10
end

-- задание параметров
function AHU_Summer:setParam(SwToSummerVal, SwToWinterVal)
    self.SwToSummerVal = SwToSummerVal
    self.SwToWinterVal = math.min(SwToWinterVal, SwToSummerVal-1)
end

function AHU_Summer:run(Md, val)
    local autVal = self.hyst:run(val, self.SwToSummerVal, self.SwToWinterVal)
    self.out = Md == 2 and ToBool(autVal) or Md==1
    return self.out
end
--*********************************************************************************************************************************************************
-- Объект. Формирование сигнала запуска установки
AHU_Start = Class()
function AHU_Start:init()
    self.prestart = false
end

function AHU_Start:run(Md, Schedule, Block)
    self.prestart = (Md == 2 and Schedule or Md == 1) and not Block
    return self.prestart
end
--*********************************************************************************************************************************************************
-- Функция. Вычисление уставок температуры обратной воды
function getWatSpt()
end
--*********************************************************************************************************************************************************
--Объект. Вычисление уставки компенсации
AHU_RatSpt = Class()
function AHU_RatSpt:init()
end

function AHU_RatSpt:setParam()
end

function AHU_RatSpt:setPid()
end

function AHU_RatSpt:run()
end
--*********************************************************************************************************************************************************
-- Объект. Выполнение предварительного прогрева водяного калорифера в режиме ЗИМА
AHU_Preheat = Class()
function AHU_Preheat:init()
    self.alarm = ALARM()
    self.rtStart = RT(2,0)
    self.prestart = false
    self.est = 0
    self.maxPreheatTm = 180
    self.prehSta = false
    self.start = false
    self.minPreheatTm = 30
end

-- задание параметров
function AHU_Preheat:setParam(minPreheatTm, maxPreheatTm)
    self.maxPreheatTm = maxPreheatTm
    self.minPreheatTm = minPreheatTm
end

function AHU_Preheat:run(prestart, RwtTmpSpt, RwtTmp, summerSta, Rst)
    self.prestart = prestart
    if ToBool(prestart) and not self.prehSta and not self.start then
        self.prehSta = true
        self.rtStart:set(0)
    end
    if not ToBool(prestart) or self.alarm.AlmSta then
        self.prehSta = false
        self.rtStart:set(0)
        self.start = false
    end
    self.rtStart:run(self.prehSta, false)
    self.est = self.rtStart.out
    if ((self.rtStart.out >= self.minPreheatTm and RwtTmp >= RwtTmpSpt) and not summerSta or summerSta) and self.prestart then
        self.prehSta = false
        self.start = true
    end
	if Rst then
		self.alarm:rst()
	end
    self.alarm:run(self.rtStart.out >= self.maxPreheatTm)
	return self.start, self.prehSta, self.alarm.AlmSta
end
--*********************************************************************************************************************************************************
-- Объект. Управление водяным калорифером
AHU_WatHtg = Class()
function AHU_WatHtg:init()
	-- входы IO
	self.Dat = 0
	self.Rwt = 0
	self.TsSta = 0
	self.PuSta = 0
	self.VlvPosSta = 0
	self.PuAutSta = 0
	self.PuHSta = 0
	-- PID
	self.pida = PID()
	self.pidw = PID()
	self.Pu = DeviceObj()
	--параметры ПИД
	self.G	= 2.0
	self.Ti = 120
	self.Td = 0.0
	self.Dz = 0.01
	self.Stroke = 60
	self.Umax = 125
	self.Umin = 0
	--Клапан
	self.Tsg = 0
	self.ValveRef = 0
	--Насос
	self.FbPuDelTm = 5
	self.MdPu = 2
	self.PuCmd = false
	self.PuSta = 0
	--Уставки защиты
	self.SptLowAirTmp = 7.0
	self.TsOpen = false
	self.prhtOpen = false
	--Аварии
	self.AL_PuFl = false
	self.AL_PuNoFbk = false
	self.AL_Ts = false
	self.AL_DatLo = false
	self.AL_RwtLo = false
	self.AL_VlvPosFl = false
	self.DelayFanWrk = DELAY()
	self.Rst = false
	
end

-- задание параметров ПИД
function AHU_WatHtg:setPid(par, Ti, Td, Dz, Stroke, FaRedEna)
	if type(par) == "table" then
		self.G = par.G==nil and self.G or par.G
		self.Ti = par.Ti==nil and self.Ti or par.Ti
		self.Td = par.Td==nil and self.Td or par.Td
		self.Dz = par.Dz==nil and self.Dz or par.Dz
		self.Stroke = par.Stroke==nil and self.Stroke or par.Stroke
		self.FaRedEna = par.FaRedEna==nil and self.FaRedEna or par.FaRedEna
	else
		self.G = par==nil and self.G or par 
		self.Ti = Ti==nil and self.Ti or Ti
		self.Td = Td==nil and self.Td or Td
		self.Dz = Dz==nil and self.Dz or Dz
		self.Stroke = Stroke==nil and self.Stroke or Stroke
		self.FaRedEna = FaRedEna==nil and self.FaRedEna or FaRedEna
	end
  if self.FaRedEna then
    self.Umax = 125
  else
    self.Umax = 100
  end
	self.pida:updatePidParam(self.G, self.Ti, self.Td, self.Dz,  self.Umax, self.Umin, self.Stroke, 1)
	self.pidw:updatePidParam(self.G, self.Ti, self.Td, self.Dz, 100, 0, self.Stroke,1)
end

function AHU_WatHtg:getIO(Dat, Rwt, TsSta, VlvPosSta, PuFbk, PuAlmSta, PuAutSta, PuHSta)
	if type(Dat) == "table" then
		self.Dat = Dat.Dat
		self.Rwt = Dat.Rwt
		self.TsSta = Dat.TsSta
		self.PuFbk = Dat.PuFbk
		self.PuAlmSta = Dat.PuAlmSta
		self.VlvPosSta = Dat.VlvPosSta
		self.PuAutSta = Dat.PuAutSta
		self.PuHSta = Dat.PuHSta
	else
		self.Dat = Dat
		self.Rwt = Rwt
		self.TsSta = TsSta
		self.PuFbk = PuFbk
		self.PuAlmSta = PuAlmSta
		self.VlvPosSta = VlvPosSta
		self.PuAutSta = PuAutSta
		self.PuHSta = PuHSta
	end
end

-- задание параметров насоса
function AHU_WatHtg:setPu(FbPuDelTm)
	self.Pu:setParam(FbPuDelTm)
	self.Pu:setAutH(self.PuAutSta, self.PuHSta)
end

-- задание уставок защиты темлообменника
function AHU_WatHtg:setSafeSet(SptLowAirTmp, FaWrkSta, CurSptRwt, SptRwtPrht, FrSta, PwrSta, Summer, Rst)
  self.CurSptRwt = CurSptRwt
  self.Summer    = Summer
  self.FaWrkSta  = FaWrkSta
	self.SptLowAirTmp = SptLowAirTmp==nil and self.SptLowAirTmp or SptLowAirTmp
	self.safetyOpen = FrSta or PwrSta or (self.AL_PuFault and not Summer)
	--контроль термостата
	if self.TsSta then
		self.AL_Ts = true
		self.TsOpen = true
	end
	if (self.Rwt > SptRwtPrht or Rst) and not self.TsSta then
		self.TsOpen = false
	end
	-- контроль низкой температуры приточного воздуха
	if self.DelayFanWrk:run(FaWrkSta, 60, 0) and self.Dat < self.SptLowAirTmp then
		self.AL_DatLo = true
    self.TsOpen = true
	end
	-- контроль низкой температуры обратной воды
	if FaWrkSta and self.Rwt < (CurSptRwt - 4) and not Summer then
		self.AL_RwtLo = true
    self.TsOpen = true
	end
	self.AL_VlvPosFl = false

	if Rst then
		self.AL_Ts = false
		self.AL_DatLo = false
		self.AL_RwtLo = false
	end
end

-- работа контура. Насос
function AHU_WatHtg:runPu(MdPu, Rst)
	local puRnPar = {}
	puRnPar.Md = MdPu
	puRnPar.AutCmd = self.Summer
	puRnPar.Fbk = self.PuFbk
	puRnPar.AlmSta = self.PuAlmSta
	puRnPar.EmrgStop = false
	puRnPar.Rst = Rst
	-- управление насосом
	self.PuCmd, self.PuSta = self.Pu:run(puRnPar)
	--Pump alarms
	self.AL_PuFault = self.Pu.AL_AlmSta.AlmSta
	self.AL_PuFbk   = self.Pu.AL_Fbk.AlmSta
end

function AHU_WatHtg:runVlv(	start,		-- сигнал запуска
							prehSta,	-- статус предварительного прогрева
							SuTmpSpt, 	-- уставка температуры притока
							ManMd, 		-- статус ручного режима управления клапаном
							ManVal 	-- ручное задание значения открытия клапана
							)
	
	if prehSta and not self.prhtOpen then
		self.prhtOpen = true
	end
	if self.prhtOpen and self.FaWrkSta or not start and not prehSta then
		self.prhtOpen = false
	end
	
	--формирование режима защитного открытия клапана
	local valveMax = self.TsOpen or self.safetyOpen or self.prhtOpen
	
	local pidaOut = self.Tsg
	local pidwOut = self.Tsg
	local moda = 0
	local modw = 0
	
	-- управление клапаном нагрева
	if ToBool(ManMd) then
		self.Tsg = ManVal
		self.ValveRef = ManVal
	else
		if self.Summer then
			self.ValveRef = 0
			self.Tsg = 0
		else
			if valveMax then
				self.ValveRef = 100
				self.Tsg = 100
			else
				if not start then
					moda = 3
				else
					moda = 1
				end
				pidaOut = self.pida:run(self.Dat,SuTmpSpt,moda,self.Tsg)
				pidwOut = self.pidw:run(self.Rwt,self.CurSptRwt,1,self.Tsg)
				self.ValveRef = math.max(pidaOut, pidwOut)
				if (self.Rwt < self.CurSptRwt-2) then
					self.ValveRef = self.ValveRef + (self.CurSptRwt-2-self.Rwt)*1.5
				end
			end
		end
	end
	self.Tsg = self.ValveRef
  if self.Tsg > 100 then
    self.FaRedVal = (self.Tsg - 100)*2
    self.ValveRef = 100
  else
    self.FaRedVal = 0
  end
end
--*********************************************************************************************************************************************************
-- Объект. Определение режима НАГРЕВ/ОХЛАЖДЕНИЕ
AHU_ClgMd = Class()
function AHU_ClgMd:init()
    self.out = false
    self.TempDelta = 2
end
-- задание параметров
function AHU_ClgMd:setParam(TempDelta)
    self.TempDelta = math.min(TempDelta, 1)
end

function AHU_ClgMd:run(Summer, SuTmpSpt, Oat, Md)
    if not Summer then
        self.out = false
    else
        if Md ~= 2 then
            self.out = ToBool(Md)
        else
            if Oat > SuTmpSpt + self.TempDelta then
                self.out = true
            elseif Oat < SuTmpSpt - self.TempDelta then
				self.out = false
            end
        end
    end
    return self.out
end
--*********************************************************************************************************************************************************
-- Объект. Охладитель с аналоговым управлением
AHU_Clg = Class()
function AHU_Clg:init()
    -- ПИД
    self.pid = PID()
    -- Настройки ПИД
    self.G = -10.0
    self.Ti = 120
    self.Td = 0.0
    self.Dz = 0.3
    self.Stroke = 10
    self.Umax = 125
    self.Umin = 0
    -- Статусы
    self.Fbk = false
    -- Клапан
    self.Tsg = 0
    self.ValveRef = 0
    -- Аварии
    self.DelayOn = DELAY()
    self.FbkDelTm = 5
    self.out = false
    self.AL_Fbk = ALARM()
	self.AL_Fbk:setParam(true, 0, true)
    self.AL_AlmSta = ALARM()
	self.AL_AlmSta:setParam(true, 2, false)
    self.Rst = false
end
-- задание параметров ПИД
function AHU_Clg:setPid(par)
    self.G = par.G == nil and self.G or par.G
    self.Ti = par.Ti == nil and self.Ti or par.Ti
    self.Td = par.Td == nil and self.Td or par.Td
    self.Dz = par.Dz == nil and self.Dz or par.Dz
    self.Stroke = par.Stroke == nil and self.Stroke or par.Stroke
    self.Umax = par.Umax == nil and self.Umax or par.Umax
    self.Umin = par.Umin == nil and self.Umin or par.Umin
    self.pid:updatePidParam(self.G, self.Ti, self.Td, self.DZ, self.Umax, self.Umin, self.Stroke, 1)
end

function AHU_Clg:getIO(Dat, VlvPosSta, Fbk, AlmSta)
	if type(Dat) == "table" then
		self.Dat = Dat.Dat
		self.Fbk = Dat.Fbk
		self.AlmSta = Dat.AlmSta
		self.VlvPosSta = Dat.VlvPosSta
	else
		self.Dat = Dat
		self.Fbk = Fbk
		self.AlmSta = AlmSta
		self.VlvPosSta = VlvPosSta
	end
end
-- основная работа регудятора
function AHU_Clg:run(start, SuTmpSpt, FaWrkSta, Summer, ManMd, ManVal, AlmSta, Rst)
    self.out = false
    self.Tsg = self.VlvPosSta
    self.ValveRef = 0
    local pidOut = self.ValveRef

	-- управление клапаном
    if ManMd then
        self.Tsg = ManVal
        self.ValveRef = ManVal
    else
        if not Summer then
            self.ValveRef = 0
            self.Tsg = 0
        else
            if not start then
                pidOut = 0
            else
                pidOut = self.pid:run(self.Dat, SuTmpSpt, 1, self.Tsg)
            end
            self.ValveRef = math.max(pidOut, 0)
        end
    end 
    -- Аварии
    self.DelayOn:run(self.out, self.FbkDelTm, 0)
    self.AL_Fbk:run(ToBool(self.DelayOn.out) and not self.Fbk)
    self.AL_AlmSta:run(AlmSta)
	local fault = self.AL_Fbk.AlmSta or self.AL_AlmSta.AlmSta
	self.out = FaWrkSta and start and (not fault)

    if Rst then
        self.AL_Fbk:rst()
        self.AL_AlmSta:rst()
    end
    return self.out
end
--*********************************************************************************************************************************************************
-- Объект. Управление группой освещения
LtGrp = Class()
function LtGrp:init()
    self.Md = 0 -- Mode: 0 - отключить, 1 - включить, 2 - автоматический режим
    self.AutCmd = false
    self.AutSta = true
    self.HSta = false
    self.Fbk = false
    self.FbkDelTm = 5
    self.AlmSta = false
    self.out = false
    self.AL_NoOn = ALARM()
	self.AL_NoOn:setParam(true, 2, true)
    self.AL_NoOff = ALARM()
	self.AL_NoOff:setParam(true, 2, true)
    self.DelayOn = DELAY()
    self.Fault = false -- статус общей аварии
    self.Sta = 0 -- статус: бит0 - статус включения, бит1 - авария
end

-- задание параметров
function LtGrp:setParam(FbkDelTm)
    self.FbkDelTm = FbkDelTm
end

function LtGrp:run(Md, AutCmd, Fbk, Rst)
    self.Md = Md
    self.AutCmd = AutCmd
    self.Fbk = Fbk
    self.DelayOn:run(self.out, self.FbkDelTm, self.FbkDelTm)
    self.AL_NoOn:run(ToBool(self.DelayOn.out) and not self.Fbk)
    self.AL_NoOff:run(not ToBool(self.DelayOn.out) and self.Fbk)
    self.Fault = self.AL_NoOn.AlmSta or self.AL_NoOff.AlmSta 
    self.Sta = 2 * ToInt(self.Fault) + 1 * ToInt(self.Fdb)
    if ToBool(Rst) then
        self:AlmRst()
    end

    self.out = (self.Md == 2 and self.AutCmd or self.Md == 1)-- and not self.Fault
    
    return self.out, self.Sta
end
--сброс аварий
function LtGrp:AlmRst()
    self.AL_NoOn:rst()
	self.AL_NoOff:rst()
end
--*********************************************************************************************************************************************************
-- Объект. Электрический нагреватель с дискретным управлением
AHU_ElHtgDig = Class()
function AHU_ElHtgDig:init()
	-- параметры
	self.StageCnt = 1
	self.TempDelta = 0.5
    -- статусы
	self.Stages = {}
	-- внутренние объекты
	self.delayUp = DELAY()
	self.delayDown = DELAY()
	self.stageUp = false
	self.stageDwn = false
	self.toStage = 0
	self.toUpTm = 15 
	self.toDwnTm = 15
    -- аварии
    self.AL_AlmSta = ALARM()
	self.AL_AlmSta:setParam(true, 2, false)
    self.Rst = false
end
-- задание параметров
function AHU_ElHtgDig:setParam(StageCnt, TempDelta, toUpTm, toDwnTm)
	self.StageCnt = StageCnt or 1
    self.TempDelta = math.min(TempDelta, 0.2)
	self.toUpTm = toUpTm or 10
	self.toDwnTm = toDwnTm or 10
end

function AHU_ElHtgDig:run(start, Dat, SuTmpSpt, FaWrkSta, Summer, ManMd, ManVal, AlmSta, Rst)
    for i = 1, self.StageCnt do
		self.Stages[i] = false
	end

    if ManMd then
        self.toStage = math.min(self.StageCnt, ManVal)
    else
		if not start then
			self.toStage = 0
		else
			if Dat < SuTmpSpt - self.TempDelta then
				if self.stageUp then 
					if self.toStage < self.StageCnt then
						self.toStage = self.toStage + 1
					end
					self.stageUp = false
				end
				self.delayUp:run(not self.stageUp, self.toUpTm, 0)
			elseif Dat > SuTmpSpt - self.TempDelta then
				if self.stageDwn then 
					if self.toStage > 0 then
						self.toStage = self.toStage - 1
					end
					self.stageDwn = false
				end
				self.delayUp:run(not self.stageUp, self.toUpTm, 0)
			else
				self.stageUp = true
				self.stageDwn = true
			end
		end
    end 
    -- аварии
    self.AL_AlmSta:run(AlmSta)
	local fault = self.AL_AlmSta.AlmSta or not FaWrkSta
	for i = 1, self.StageCnt do
		if i <= self.toStage then self.Stages[i] = true and (not fault) else self.Stages[i] = false end
	end
	self.out = FaWrkSta and start and (not fault)

    if Rst then
        self.AL_AlmSta:rst()
    end
    return self.Stages
end
--*********************************************************************************************************************************************************
-- Объект. Управление электрическим нагревателем с аналоговым управлением
AHU_ElHtgAn = Class()
function AHU_ElHtgAn:init()
	-- ПИД
	self.pida = PID()
	--Настройки ПИД
	self.G	= 0.7
	self.Ti = 120
	self.Td = 0.1
	self.Dz = 0.3
	self.Stroke = 0
	self.Umax = 125
	self.Umin = 0
	--Выход
	self.Tsg = 0
	self.ValveRef = 0
	--Настройки защиты
	self.TsOpen = false
	--Аварии
	self.AL_AlmSta = ALARM()
	self.AL_AlmSta:setParam(true, 2, false)
	self.AL_HtgNoOn = ALARM()
	self.AL_HtgNoOn:setParam(true, 2, true)
	self.AL_HtgNoOff = ALARM()
	self.AL_HtgNoOff:setParam(true, 2, true)
	self.AL_TS = false
	self.AL_LowAirTmp = false
	self.DelayFanWrk = DELAY()
	self.Rst = false
end
-- задание параметров ПИД
function AHU_ElHtgAn:setPid(par)
	self.G = par.G==nil and self.G or par.G
	self.Ti = par.Ti==nil and self.Ti or par.Ti
	self.Td = par.Td==nil and self.Td or par.Td
	self.Dz = par.Dz==nil and self.Dz or par.Dz
	self.Stroke = par.Stroke==nil and self.Stroke or par.Stroke
	self.Umax = par.Umax==nil and self.Umax or par.Umax
	self.Umin = par.Umin==nil and self.Umin or par.Umin
	self.pida:updatePidParam(self.G, self.Ti, self.Td, self.Umax, self.Umin, self.Stroke, 1)
end
--задание уставок защиты
function AHU_ElHtgAn:setSafeSet(SptLowAirTmp, TsSta, FrSta, PwrSta)
	self.SptLowAirTmp = SptLowAirTmp==nil and self.SptLowAirTmp or SptLowAirTmp
	self.safetyOff = TsSta or FrSta or PwrSta
end
-- основная работа
function AHU_ElHtgAn:run(start, Dat, SuTmpSpt, FaWrkSta, curVal, Summer, ManMd, ManVal, HtgFbk, HtgAlm, Rst)
	self.Tsg = curVal
	
	if self.safetyOff or self.AL_AlmSta.AlmSta then 
		self.Tsg = 0
		self.HtgCmd = false
		self.HtgRef = 0
	else
		local pidaOut = self.Tsg
		
		if ToBool(ManMd) then
			self.Tsg = ManVal
			self.HtgRef = ManVal
			self.HtgCmd = self.HtgRef > 0
		else
			if Summer then
				self.HtgRef = 0
				self.HtgCmd = false
				self.Tsg = 0
			else
				if not start or not FaWrkSta then
					pidaOut = 0
					self.HtgCmd = false
				else
					pidaOut = self.pida:run(Dat,SuTmpSpt,1,self.Dz,self.Tsg,1)
					self.HtgCmd = true
				end
				self.HtgRef = pidaOut
			end
		end
	end
	--Аварии
	self.AL_HtgNoOn:run(ToBool(self.HtgCmd) and not HtgFbk)
	self.AL_HtgNoOff:run(ToBool(not self.HtgCmd) and HtgFbk)
	self.AL_AlmSta:run(HtgAlm)
	--Термостат
	if self.TsSta then
		self.AL_TS = true
	end
	-- Низкая температура воздуха
	if self.DelayFanWrk:run(FaWrkSta, 60, 0) and Dat < self.SptLowAirTmp then
		self.AL_LowAirTmp = true
	end
	
	if Rst then
		self.AL_HtgNoOn:rst()
		self.AL_HtgNoOff:rst()
		self.AL_AlmSta:rst()
		self.AL_TS = false
		self.AL_LowAirTmp = false
	end
end
--*********************************************************************************************************************************************************
-- Объект. Базовое управление рекуператором
AHU_Recup = Class()
function AHU_Recup:init()
--наполнение для конструктора
end

function AHU_Recup:setParam()
	-- наполнение для метода
end

-- Контроль обмерзания рекуператора. Определение по реле перепада давления
function AHU_Recup:getFrzAlmPds()
--наполнение для метода
end

-- Контроль обмерзания рекуператора. Определение по перепаду давления
function AHU_Recup:getFrzAlmPde()
	--наполнение для метода
end

-- Контроль обмерзания рекуператора. Определение по снижению КПД
function AHU_Recup:getFrzAlmKpd()
	--наполнение для метода
end

-- Контроль обмерзания рекуператора. Определение по падению температуры выбрасываемого воздуха
function AHU_Recup:getFrzAlmEat()
	--наполнение для метода
end

function AHU_Recup:run()
	--наполнение для метода
end
--*********************************************************************************************************************************************************
-- Объект. Управление рекуператором с байпасом с дискретным управлением
AHU_RecupDig = Class(AHU_Recup)
function AHU_RecupDig:init()
--наполнение для конструктора
end

function AHU_RecupDig:setParam()
	-- наполнение для метода
end

function AHU_RecupDig:run()
	--наполнение для метода
end
--*********************************************************************************************************************************************************
-- Объект. Управление рекуператором с байпасом с аналоговым управлением
AHU_RecupAn = Class(AHU_Recup)
function AHU_RecupAn:init()
--наполнение для конструктора
end

function AHU_RecupAn:setParam()
	-- наполнение для метода
end

-- задание параметров ПИД
function AHU_RecupAn:setPid()
  end

function AHU_RecupAn:run()
	--наполнение для метода
end
--*********************************************************************************************************************************************************
--- Объект. Работа с влажностью
AHU_HumCtrl = Class()
function AHU_HumCtrl:init()
	--наполнение для метода
end
-- вычисление абсолютной влажности
function AHU_HumCtrl:getAbsHum()
end
-- конвертация уставки относительной влажности в единицы абсолютной влажности
function AHU_HumCtrl:convertSpt()
end
-- конвертация текущей влажности притока в единицы абсолютной влажности
function AHU_HumCtrl:convertSupHum()
end
-- конвертация текущей влажности вытяжки в единицы абсолютной влажности
function AHU_HumCtrl:convertExHum()
end
--задание параметров
function AHU_HumCtrl:setParam()
end
--*********************************************************************************************************************************************************
-- Объект. Управление увлажнителем с дискретным управлением
AHU_HumCtrlDig = Class(AHU_HumCtrl)
function AHU_HumCtrlDig:inti()
end

-- задание параметров
function AHU_HumCtrlDig:setParam()
end

-- основная работа
function AHU_HumCtrlDig:run()
end
--*********************************************************************************************************************************************************
-- Объект. Работа увлажнителя с аналоговым управлением
AHU_HumCtrlAn = Class(AHU_HumCtrl)

function AHU_HumCtrlAn:init()
end

function AHU_HumCtrlAn:setPid()
end

function AHU_HumCtrlAn:setParam()
end

function AHU_HumCtrlAn:run()
end
--*********************************************************************************************************************************************************
-- Объект. Управление осушением
AHU_HumDry = Class(AHU_HumCtrl)
function AHU_HumDry:init()
end

function AHU_HumDry:setParam()
end

function AHU_HumDry:setPid()
end

function AHU_HumDry:run()
end
--*********************************************************************************************************************************************************
-- Объект. Управление рециркуляцией
AHU_RecircAn = Class()
function AHU_RecircAn:init()
end

function AHU_RecircAn:setParam()
end

function AHU_RecircAn:setPid()
end
--*********************************************************************************************************************************************************